"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.extractReferencedIdentifiers = extractReferencedIdentifiers;
exports.getAssignmentCode = getAssignmentCode;
exports.hasInVariables = hasInVariables;
var _parser = require("@babel/parser");
var _traverse = _interopRequireDefault(require("@babel/traverse"));
/**
 * 从 JS 表达式中提取被引用的变量标识符（Identifier）
 * @param {string} code JS 表达式，如 `test === 't' && test2 === 'b'`
 * @returns {string[]} 被引用的变量名数组
 */
function extractReferencedIdentifiers(code) {
  var ast = (0, _parser.parse)(code, {
    sourceType: 'module',
    plugins: ['jsx', 'typescript'] // 可根据需要添加
  });
  var identifiers = new Set();
  (0, _traverse["default"])(ast, {
    Identifier: function Identifier(path) {
      var node = path.node,
        parent = path.parent,
        parentPath = path.parentPath;

      // 1. 忽略变量声明（如 let a = 1 中的 a）
      if (parent.type === 'VariableDeclarator' && parent.id === node) return;

      // 2. 忽略函数定义名 function a() {}
      if (parent.type === 'FunctionDeclaration' && parent.id === node) return;

      // 3. 忽略函数参数 function(a) {}
      if (parent.type === 'FunctionExpression' || parent.type === 'ArrowFunctionExpression') {
        if (parent.params.includes(node)) return;
      }

      // 4. 忽略对象属性名 { a: 123 }
      if (parent.type === 'ObjectProperty' && parent.key === node && !parent.computed) return;

      // 5. 忽略 class 定义名
      if (parent.type === 'ClassDeclaration' && parent.id === node) return;

      // 6. 如果是 MemberExpression 的属性，如 obj.test 中的 `test`，且非 computed，不算变量引用
      if (parent.type === 'MemberExpression' && parent.property === node && !parent.computed) return;

      // 7. 只保留真实的引用
      if (path.isReferencedIdentifier()) {
        identifiers.add(node.name);
      }
    }
  });
  return [].concat(identifiers);
}

/**
 * 提取变量赋值语句中 `=` 号右边的表达式字符串
 * @param {string} code - 代码字符串，如 `const test = currentUser === 'admin';`
 * @returns {string | null} - 返回右侧表达式字符串（如 `currentUser === 'admin'`），如果无效则为 null
 */
function getAssignmentCode(code) {
  var rightExpression = null;
  try {
    var ast = (0, _parser.parse)(code, {
      sourceType: 'module',
      plugins: ['jsx', 'typescript'],
      ranges: true,
      tokens: true
    });
    (0, _traverse["default"])(ast, {
      VariableDeclarator: function VariableDeclarator(path) {
        var _path$node;
        var init = (_path$node = path.node) === null || _path$node === void 0 ? void 0 : _path$node.init;
        if (init && init.start != null && init.end != null) {
          rightExpression = code.slice(init.start, init.end);
        }
      }
    });
  } catch (err) {
    console.error('Failed to parse code:', err.message);
  }
  return rightExpression;
}

// 获取所有 children 节点组成的扁平数组
function collectAllChildren(tree) {
  var result = [];
  function traverse(nodes) {
    nodes.forEach(function (node) {
      if (node.children) {
        result.push.apply(result, node.children);
        traverse(node.children); // 递归处理嵌套的 children
      }
    });
  }
  traverse(tree);
  return result;
}

// 判断 identifier 是否存在
function hasInVariables(variables, identifier) {
  var allChildren = collectAllChildren(variables);
  return allChildren.some(function (child) {
    return child.key === identifier;
  });
}